library StatusLog;

uses
  Windows,
  SysUtils,
  Classes,
  DateUtils,
  m_api;

var
  PluginInterfaces: array[0..1] of MUUID;
  {OnLoadHook,} OnSettingsHook: THANDLE;
  //Protos: TStringList;

function StatusToStr(const Status: Word): string;
begin
  case Status of
    ID_STATUS_OFFLINE: Result:= TranslateUnicodeString('Offline');
    ID_STATUS_ONLINE: Result:= TranslateUnicodeString('Online');
    ID_STATUS_AWAY: Result:= TranslateUnicodeString('Away');
    ID_STATUS_DND: Result:= TranslateUnicodeString('DND');
    ID_STATUS_NA: Result:= TranslateUnicodeString('NA');
    ID_STATUS_OCCUPIED: Result:= TranslateUnicodeString('Occupied');
    ID_STATUS_FREECHAT: Result:= TranslateUnicodeString('Free for chat');
    ID_STATUS_INVISIBLE: Result:= TranslateUnicodeString('Invisible');
  else
    Result:= TranslateUnicodeString('Unknown');
  end;
end;

function UnixTimeStamp: Cardinal;
var
  TZI: TTimeZoneInformation;
begin
  GetTimeZoneInformation(TZI);
  Result:= DateTimeToUnix(Now)+TZI.Bias*60;
end;

function OnContactSettingsChanged(wParam: WPARAM; lParam: LPARAM): int; cdecl;
var
  Settings: PDBCONTACTWRITESETTING;
  NewStatus, OldStatus: Word;
  EI: TDBEVENTINFO;
  Text: UTF8String;
begin
  if (wParam <> 0) and (lParam <> 0) then
    begin
      Settings:= PDBCONTACTWRITESETTING(lParam);
      if (AnsiString(Settings^.szSetting) = 'Status') and
         //(Protos.IndexOf(string(Settings^.szModule)) >= 0) and
         (AnsiString(PAnsiChar(CallService(MS_PROTO_GETCONTACTBASEPROTO, wParam, 0))) = AnsiString(Settings^.szModule)) and
         (Settings^.value._type = DBVT_WORD) and
         (Settings^.value.wVal >= ID_STATUS_OFFLINE) and
         (Settings^.value.wVal <= ID_STATUS_INVISIBLE) and
         (CallService(MS_IGNORE_ISIGNORED, wParam, IGNOREEVENT_USERONLINE) = 0) then
        begin
          NewStatus:= Settings^.value.wVal;
          OldStatus:= DBGetContactSettingWord(wParam, Settings^.szModule, 'OldStatus', ID_STATUS_OFFLINE);
          if OldStatus <> NewStatus then
            begin
              DBWriteContactSettingWord(wParam, Settings^.szModule, 'OldStatus', NewStatus);

              Text:= UTF8String(Format(TranslateUnicodeString('[Status changed from %s to %s]'), [StatusToStr(OldStatus), StatusToStr(NewStatus)]));

              EI.cbSize:= SizeOf(EI);
              EI.szModule:= Settings^.szModule;
              EI.timestamp:= UnixTimeStamp;
              EI.flags:= DBEF_READ or DBEF_UTF;
              EI.eventType:= EVENTTYPE_MESSAGE;
              EI.cbBlob:= Length(Text)+1;
              EI.pBlob:= PByte(Addr(Text[1]));
              db_event_add(wParam, @EI);
            end;
        end;
    end;
  Result:= 0;
end;

function MirandaPluginInfoEx(mirandaVersion:DWORD): PPLUGININFOEX; cdecl;
const
  UUID: MUUID = '{1E29B30B-6660-45DC-B0A1-F60293605CC4}';
begin
  PluginInfo.cbSize:= SizeOf(PluginInfo);
  PluginInfo.shortName:= 'TrackUserStatus';
  PluginInfo.version:= $00000001;
  PluginInfo.description:= 'Track user status changes to history';
  PluginInfo.author:= 'quwy';
  PluginInfo.authorEmail:= 'quwy@ukr.net';
  PluginInfo.copyright:= 'quwy (c) 2014';
  PluginInfo.homepage:= 'http://miranda-icq.sourceforge.net/';
  PluginInfo.flags:= UNICODE_AWARE;
  PluginInfo.uuid:= UUID;
  Result:= @PluginInfo;
end;

{function OnModulesLoaded(wParam: WPARAM; lParam: LPARAM): int; cdecl;
begin
  UnhookEvent(OnLoadHook);

  Result:= 0;
end;}

function Load: int; cdecl;
begin
  Langpack_Register;

  //Protos:= TStringList.Create;
  //Protos.Sorted:= True;

  //OnLoadHook:= HookEvent(ME_SYSTEM_MODULESLOADED, OnModulesLoaded);
  OnSettingsHook:= HookEvent(ME_DB_CONTACT_SETTINGCHANGED, OnContactSettingsChanged);

  Result:=0;
end;

function Unload: int; cdecl;
begin
  UnhookEvent(OnSettingsHook);

  //Protos.Free;

  Result:= 0;
end;

function MirandaPluginInterfaces: PMUUID; cdecl;
begin
  PluginInterfaces[0]:= MIID_TESTPLUGIN;
  PluginInterfaces[1]:= MIID_LAST;
  Result:= @PluginInterfaces;
end;

exports
  Load, Unload, MirandaPluginInterfaces, MirandaPluginInfoEx;

begin

end.

